@@ -2,9 +2,16 @@ use crate::colorscheme::ToForeground;
 use crate::langs;
 use crate::settings::TestSummary;
 use crate::settings::TypingTestConfig;
-use std::time::Instant;
+use std::time::{Duration, Instant};
 use tui::{style::Color, text::Span};
 
+/// 0.05s tiny time offset appllied when pressed the first key
+/// 0.05s delay between chars corresponds to 240wpm
+/// which is more than the world record
+/// seems more than fair
+const INITAL_OFFSET: Duration = Duration::from_millis(50);
+const MAX_EXTRA_MISTAKES: usize = 3;
+
 pub struct WpmHoarder {
     pub wpms: Vec<f64>,
     pub capacity: usize,
@@ -53,6 +60,20 @@ impl WpmHoarder {
         }
         max
     }
+
+    pub fn get_min_max_wpm(&self) -> (f64, f64) {
+        let mut max: f64 = self.wpms[0];
+        let mut min: f64 = self.wpms[0];
+
+        for wpm in &self.wpms[1..] {
+            if *wpm > max {
+                max = *wpm
+            } else if *wpm < min {
+                min = *wpm
+            }
+        }
+        (min, max)
+    }
 }
 
 pub struct TestColors {
@@ -88,11 +109,13 @@ pub struct TestState<'a> {
     pub blanks: usize,
 
     pub mistakes: usize,
+    pub extra_mistakes: usize,
     pub pmiss: usize,
 
     pub cursor_x: u16,
     pub current_char: char,
 
+    pub first: bool,
     pub begining: Instant,
     // source for generating test
     pub source: String,
@@ -119,12 +142,14 @@ impl<'a> Default for TestState<'a> {
             // characters done on current line
             // this variable is reset after each line
             done: 0,
+            first: true,
 
             // chars done on previous lines
             // p stands for persistent I guess
             pdone: 0,
             blanks: 0,
             mistakes: 0,
+            extra_mistakes: 0,
 
             // persistent mistakes - these cant be backspaced away
             // used to calculate accuracy
@@ -135,7 +160,7 @@ impl<'a> Default for TestState<'a> {
             source: "storage/words/english".to_string(),
             length: 0,
             current_char: ' ',
-            hoarder: WpmHoarder::new(32),
+            hoarder: WpmHoarder::new(400),
             colors: TestColors::default(),
         }
     }
@@ -164,7 +189,7 @@ impl<'a> TestState<'a> {
     pub fn summarize(&self) -> TestSummary {
         TestSummary {
             correct_chars: self.pdone + self.done - self.blanks - self.mistakes,
-            mistakes: self.mistakes,
+            mistakes: self.mistakes + self.extra_mistakes,
             wpm: self.calculate_wpm(),
             acc: self.calculate_acc(),
         }
@@ -177,6 +202,7 @@ impl<'a> TestState<'a> {
         self.up = vec![];
         self.pmiss = 0;
         self.mistakes = 0;
+        self.extra_mistakes = 0;
         self.hoarder.reset();
 
         let mut wordy = langs::prepare_test(config, &self.colors);
@@ -186,6 +212,7 @@ impl<'a> TestState<'a> {
         self.backburner = wordy;
         self.current_char = self.active[self.done].content.chars().next().unwrap();
         self.length = self.active.len();
+        self.first = true;
         self.begining = Instant::now();
     }
 
@@ -292,6 +319,22 @@ impl<'a> TestState<'a> {
     /// returns true when the test is done
     pub fn on_char(&mut self, c: char) -> bool {
         self.cursor_x += 1;
+
+        // TODO this implemenation is quick and dirty
+        // and is just slapped onto existing infrastracture
+        // I don't really care for now
+        // as it works
+        if self.first {
+            self.hoarder.reset();
+            self.first = false;
+            // ofseting begining prevents very ugly graph start though not entirely.
+            // It's also only fair as the player would get the first char for free
+            // instead he gets it on the above world record pace of 240wpm
+            // which isn't at all significant for the player
+            // but helps the software
+            self.begining = Instant::now().checked_sub(INITAL_OFFSET).unwrap();
+        }
+
         if c == self.current_char {
             self.active[self.done].style = self.colors.done.fg();
             self.done += 1;
@@ -301,14 +344,17 @@ impl<'a> TestState<'a> {
         // wrong key
         // adds the mistake and the end of the word
         if self.current_char == ' ' {
-            // doesnt count as mistake
-            // but maybe as some sort of extra
             self.pmiss += 1;
-            if self.fetch(self.done - 1).len() < 3 {
+            if self.fetch(self.done - 1).len() < MAX_EXTRA_MISTAKES {
+                self.extra_mistakes += 1;
                 self.active[self.done - 1].content.to_mut().push(c);
             } else {
                 // cursor is pushed +1 when KeyCode::Char is matched
-                // well in this rare case nothing happens so it needs to revert
+                // well in this rare case nothing happens so it needs to revert;
+                // $extra_mistakes is not incremented because the mistake itself
+                // isn't shown on the screen after exceeding the limit.
+                // the pmiss is bumped so it still contributes to the accuracy though
+                // TODO extremely low priority "controversial" decision to think through
                 self.cursor_x -= 1;
             }
         // just changes to wrong and moves on
@@ -327,7 +373,10 @@ impl<'a> TestState<'a> {
     // undo word
 
     fn undo_space_char_and_extras(&mut self) {
-        self.cursor_x -= self.fetch(self.done - 1).len() as u16 + 1;
+        let x = self.fetch(self.done - 1).len();
+        self.extra_mistakes -= x;
+        debug!("{}", x);
+        self.cursor_x -= x as u16 + 1;
         self.change(self.done - 1, String::new());
         self.done -= 2;
 
@@ -383,6 +432,7 @@ impl<'a> TestState<'a> {
                         .to_mut()
                         .pop()
                         .expect("checked above");
+                    self.extra_mistakes -= 1;
                 }
             } else {
                 self.done -= 1;
